@@ -158,7 +158,7 @@ bool KdePlatformDependent::saveCredentials(const QUrl & /*baseUrl*/, const QStri
     // directly requests the accounts kcm to start adding a new account if it's not there, maybe even pre-fills the fields...)
 
     KCMultiDialog *dialog = new KCMultiDialog;
-    dialog->addModule(QStringLiteral("kcm_kaccounts"));
+    dialog->addModule(KPluginMetaData(QStringLiteral("kcm_kaccounts")));
     dialog->setAttribute(Qt::WA_DeleteOnClose);
     dialog->show();
 
@@ -193,9 +193,9 @@ bool Attica::KdePlatformDependent::askForCredentials(const QUrl &baseUrl, QStrin
 QList<QUrl> KdePlatformDependent::getDefaultProviderFiles() const
 {
     KConfigGroup group(m_config, "General");
-    QStringList pathStrings = group.readPathEntry("providerFiles", QStringList(QStringLiteral("https://autoconfig.kde.org/ocs/providers.xml")));
+    const QStringList pathStrings = group.readPathEntry("providerFiles", QStringList(QStringLiteral("https://autoconfig.kde.org/ocs/providers.xml")));
     QList<QUrl> paths;
-    foreach (const QString &pathString, pathStrings) {
+    for (const QString &pathString : pathStrings) {
         paths.append(QUrl(pathString));
     }
     qCDebug(ATTICA_PLUGIN_LOG) << "Loaded paths from config:" << paths;
