@@ -172,7 +172,7 @@ int Positioner::nearestItem(int currentIndex, Qt::ArrowType direction)
     int lastDistance = -1;
     int distance = 0;
 
-    foreach (int row, rows) {
+    for (const int row : std::as_const(rows)) {
         const QPoint pos(row % m_perStripe, row / m_perStripe);
 
         if (row == currentIndex) {
@@ -345,17 +345,17 @@ void Positioner::reset()
     Q_EMIT positionsChanged();
 }
 
-void Positioner::move(const QVariantList &moves)
+int Positioner::move(const QVariantList &moves)
 {
     // Don't allow moves while listing.
     if (m_folderModel->status() == FolderModel::Listing) {
         m_deferMovePositions = moves;
-        return;
+        return -1;
     }
 
     QVector<int> fromIndices;
     QVector<int> toIndices;
-    QVector<int> sourceRows;
+    QVariantList sourceRows;
 
     for (int i = 0; i < moves.count(); ++i) {
         const int isFrom = (i % 2 == 0);
@@ -377,7 +377,7 @@ void Positioner::move(const QVariantList &moves)
     for (int i = 0; i < fromIndices.count(); ++i) {
         const int from = fromIndices[i];
         int to = toIndices[i];
-        const int sourceRow = sourceRows[i];
+        const int sourceRow = sourceRows[i].toInt();
 
         if (sourceRow == -1 || from == to) {
             continue;
@@ -433,7 +433,11 @@ void Positioner::move(const QVariantList &moves)
         endRemoveRows();
     }
 
+    m_folderModel->updateSelection(sourceRows, true);
+
     m_updatePositionsTimer->start();
+
+    return toIndices.constFirst();
 }
 
 void Positioner::updatePositions()
@@ -898,9 +902,9 @@ void Positioner::flushPendingChanges()
 
     int last = lastRow();
 
-    foreach (const QModelIndex &idx, m_pendingChanges) {
-        if (idx.row() <= last) {
-            Q_EMIT dataChanged(idx, idx);
+    for (const QModelIndex &index : std::as_const(m_pendingChanges)) {
+        if (index.row() <= last) {
+            Q_EMIT dataChanged(index, index);
         }
     }
 
